#! /usr/bin/env python3.3
"""Git Fusion package constants."""

import os

# pylint:disable=line-too-long

# -----------------------------------------------------------------------------
#                 Begin block copied to both p4gf_const.py
#                 and p4gf_submit_trigger.py.
#
# Normal usage of Git Fusion should not require changing of the
# P4GF_DEPOT constant. If a site requires a different depot name
# then set this constant on ALL Git Fusion instances to the same
# depot name.
#
# This depot should be created by hand prior to running any Git
# Fusion instance. Wild card and revision characters are not
# allowed in depot names (*, ..., @, #) and non-alphanumeric
# should typically be avoided.

P4GF_DEPOT = '.git-fusion'
P4GF_DEPOT_NAME = 'P4GF_DEPOT'
P4GF_P4KEY_P4GF_DEPOT = 'git-fusion-p4gf-depot'
P4GF_REQ_PATCH = {
    2015.1: 1171507,
    2015.2: 1171507
}
#
#                 End block copied to both p4gf_const.py
#                 and p4gf_submit_trigger.py.
# -----------------------------------------------------------------------------

P4GF_CLIENT_PREFIX = "git-fusion-"
P4GF_REPO_CLIENT = "git-fusion-{server_id}-{repo_name}"
P4GF_REPO_CLIENT_UNIQUE = "git-fusion-{repo_name}-{uuid}"
P4GF_REPO_TEMP_CLIENT = "git-fusion--temp-{server_id}-{repo_name}-{uuid}"
P4GF_REPO_READ_PERM_CLIENT = "git-fusion--readperm-{server_id}-{repo_name}-{uuid}"
P4GF_CONFIG_VALIDATE_CLIENT = "git-fusion--config-validate-{uuid}"
P4GF_SWARM_CLIENT = "git-fusion--{repo_name}-swarm"
P4GF_OBJECT_CLIENT_PREFIX = "git-fusion--"
P4GF_OBJECT_CLIENT_12_2 = "git-fusion--{hostname}"
P4GF_OBJECT_CLIENT = "git-fusion--{server_id}"
P4GF_OBJECT_CLIENT_UNIQUE = "git-fusion--{server_id}-{uuid}"
P4GF_GROUP = "git-fusion-group"
P4GF_USER = "git-fusion-user"
P4GF_UNKNOWN_USER = "unknown_git"

P4GF_GROUP_REPO_PULL = "git-fusion-{repo_name}-pull"
P4GF_GROUP_REPO_PUSH = "git-fusion-{repo_name}-push"
P4GF_GROUP_PULL = "git-fusion-pull"
P4GF_GROUP_PUSH = "git-fusion-push"
P4GF_GROUP_ADMIN = "git-fusion-admin"

# Use hyphens, not underscores.
P4GF_P4KEY_INIT_STARTED = 'git-fusion-init-started'
P4GF_P4KEY_INIT_COMPLETE = 'git-fusion-init-complete'
P4GF_P4KEY_UPGRADE_STARTED = 'git-fusion-upgrade-started'
P4GF_P4KEY_UPGRADE_COMPLETE = 'git-fusion-upgrade-complete'
P4GF_P4KEY_PERMISSION_GROUP_DEFAULT = 'git-fusion-permission-group-default'
P4GF_P4KEY_PUSH_STARTED = 'git-fusion-{repo_name}-push-start'
P4GF_P4KEY_LAST_COPIED_CHANGE = \
    'git-fusion-{repo_name}-{server_id}-last-copied-changelist-number'
P4GF_P4KEY_LAST_SEEN_CHANGE = \
    'git-fusion-{repo_name}-{server_id}-last-seen-changelist-number'
P4GF_P4KEY_PRE_P2G_LAST_COPIED_CHANGE =  \
    'git-fusion-{repo_name}-{push_id}-pre-p2g-last-copied-changelist-number'
P4GF_P4KEY_REPO_SERVER_CONFIG_REV = 'git-fusion-{repo_name}-{server_id}-p4gf-config-rev'
P4GF_P4KEY_PERM_CHECK = 'git-fusion-auth-server-perm-check'
# -----------------------------------------------------------------------------
#                 Begin block copied to both p4gf_const.py
#                 and p4gf_submit_trigger.py.
#
P4GF_P4KEY_LOCK_VIEW = 'git-fusion-view-{repo_name}-lock'
P4GF_P4KEY_LOCK_VIEW_OWNERS = 'git-fusion-view-{repo_name}-lock-owners'
#
#                 End block copied to both p4gf_const.py
#                 and p4gf_submit_trigger.py.
# -----------------------------------------------------------------------------
P4GF_P4KEY_SERVER_ID = 'git-fusion-server-id-'
P4GF_P4KEY_LOCK_SPACE = 'git-fusion-space-lock'

P4GF_P4KEY_PENDING_MB = 'git-fusion-view-{repo_name}-pending-mb'
P4GF_P4KEY_TOTAL_MB = 'git-fusion-view-{repo_name}-total-mb'

P4GF_P4KEY_ALL_PENDING_MB = 'git-fusion-space-pending-mb'
P4GF_P4KEY_ALL_REMAINING_MB = 'git-fusion-space-remaining-mb'

P4GF_P4KEY_UPDATE_AUTH_KEYS = 'git-fusion-auth-keys-last-changenum-{}'

# Needed to check for an un-upgraded 2012.2 install
P4GF_P4KEY_OLD_UPDATE_AUTH_KEYS = 'p4gf_auth_keys_last_changenum-{}'

P4GF_P4KEY_TIME_ZONE_NAME = 'git-fusion-perforce-time-zone-name'

P4GF_P4KEY_PREVENT_SESSIONS = 'git-fusion-prevent-sessions-'
P4GF_P4KEY_PREVENT_SESSIONS_ALL = 'git-fusion-prevent-sessions-all'
P4GF_P4KEY_PREVENT_SESSIONS_INSTANCE = 'git-fusion-prevent-sessions-instance--'
P4GF_P4KEY_PREVENT_SESSIONS_REPO = 'git-fusion-prevent-sessions-repo--'
P4GF_P4KEY_PREVENT_SESSIONS_INSTANCE_REPO = \
    'git-fusion-prevent-sessions-instance-repo--{server_id}--{repo_name}'
P4GF_P4KEY_DISABLE_NON_FAST_FORWARD = 'git-fusion-disable-non-fastforward'
P4GF_P4KEY_LAST_COPIED_TAG = 'git-fusion-{repo_name}-{server_id}-last-copied-tag'
P4GF_P4KEY_READ_PERMISSION_CHECK = 'git-fusion-read-permission-check'
P4GF_P4KEY_DISABLE_ERROR_CLEANUP = 'git-fusion-disable-error-cleanup'
P4GF_P4KEY_REV_SHA1 = 'git-fusion-{repo_name}-rev-sha1-{change_num}'
P4GF_P4KEY_CHECKPOINT = 'git-fusion-checkpoint-{repo_name}-{server_id}-{name}'
P4GF_P4KEY_REPO_STATUS = 'git-fusion-status-{repo_name}'
P4GF_P4KEY_REPO_STATUS_PUSH_ID = 'git-fusion-status-{repo_name}-{push_id}'
P4GF_P4KEY_REPO_PUSH_ID = 'git-fusion-push-id-{repo_name}'
P4GF_P4KEY_PROXY_PROTECTS = 'git-fusion-proxy-protects'
P4GF_P4KEY_MIRROR_STATUS = 'git-fusion-mirror-{repo_name}-{server_id}'


# for shorter names: OT = Object Type
#                    LCN = Last Change Num
P4GF_P4KEY_INDEX_OT = \
    'git-fusion-index-branch-{repo_name},{change_num},{branch_id}'
P4GF_P4KEY_INDEX_LCN_ON_BRANCH = 'git-fusion-index-last-{repo_name},{branch_id}'
P4GF_P4KEY_INDEX_LCN_ON_BRANCH_REPO_ALL = 'git-fusion-index-last-{repo_name},*'

P4GF_BRANCH_EMPTY_REPO = 'p4gf_empty_repo'
P4GF_BRANCH_TEMP_N = 'git-fusion-temp-branch-{}'

# Environment vars
P4GF_AUTH_P4USER = 'P4GF_AUTH_P4USER'
P4GF_FOREGROUND_PUSH = 'P4GF_FOREGROUND_PUSH'
P4GF_FORK_PUSH = 'P4GF_FORK_PUSH'
P4GF_FORUSER = 'P4GF_FORUSER'

# Internal debugging keys
# section in rc file for test vars
P4GF_TEST = 'test'

# Assign sequential UUID numbers so that test scripts
# get the same results every time.
P4GF_TEST_UUID_SEQUENTIAL = 'P4GF_TEST_UUID_SEQUENTIAL'

VIRT_STREAM_SUFFIX = "_p4gfv"

# Internal testing environment variables.
# Read config from here, not /etc/git-fusion.log.conf
P4GF_TEST_LOG_CONFIG_PATH = 'P4GF_LOG_CONFIG_FILE'
P4GF_TEST_CHECK_REVIEWS_USERS_LOGIN = 'P4GF_TEST_CHECK_REVIEWS_USERS_LOGIN'
# Setting this per-repo environment variable will:
#   1. log a stack trace of each RepoLock.acquire().
#   2. increment a counter: P4GF_P4KEY_LOCK_VIEW+"_counter"
P4GF_TEST_LOG_REPO_LOCK = 'P4GF_TEST_LOG_REPO_LOCK_{repo_name}'

# Logging related environment variables.
P4GF_LOG_UUID = 'P4GF_LOG_UUID'
P4GF_LOG_REPO = 'P4GF_LOG_REPO'
P4GF_LOG_REPO_NAME = 'P4GF_LOG_REPO_NAME'
P4GF_LOGS_DIR = 'P4GF_LOGS_DIR'

# Set the LFS_FILE_MAX_SECONDS_TO_KEEP from the environment
P4GF_TEST_LFS_FILE_MAX_SECONDS_TO_KEEP = 'LFS_FILE_MAX_SECONDS_TO_KEEP'

# Label/tag added to .gitmodules file to indicate a submodule that is
# managed by Git Fusion via the stream-imports-as-submodules feature.
P4GF_MODULE_TAG = 'p4gf'

# Filenames
P4GF_DIR = '.git-fusion'
P4GF_ID_FILE = 'server-id'
P4GF_MOTD_FILE = '{P4GF_DIR}/motd.txt'
P4GF_FAILURE_LOG = '{P4GF_DIR}/logs/{prefix}{date}.log.txt'
P4GF_SWARM_PRT = 'swarm-pre-receive-list'
P4GF_NEW_DEPOT_BRANCH_LIST = 'new-depot-branch-list'
P4GF_PRE_RECEIVE_FLAG = 'pre-receive-flag'

# P4GF_HOME
P4GF_HOME = os.path.expanduser(os.path.join("~", P4GF_DIR))
P4GF_HOME_NAME = 'P4GF_HOME'

# In support of the P4GF_ENV configuration
P4GF_ENV = None               # set from the env var P4GF_ENV, if it exists
P4GF_ENV_NAME = 'P4GF_ENV'
READ_ONLY = False
READ_ONLY_NAME = 'READ_ONLY'
MAX_TEMP_CLIENTS = 10
MAX_TEMP_CLIENTS_NAME = 'MAX_TEMP_CLIENTS'
GIT_BIN_DEFAULT = 'git'
GIT_BIN_NAME = 'GIT_BIN'
GIT_BIN = GIT_BIN_DEFAULT

# section definition here avoids circularity issues with p4gf_env_config and p4gf_config
SECTION_ENVIRONMENT = 'environment'

# Perforce copies of Git commit and ls-tree objects live under this root.
P4GF_OBJECTS_ROOT = '//{P4GF_DEPOT}/objects'

# Config files (stored in Perforce, not local filesystem)
P4GF_CONFIG_GLOBAL = '//{P4GF_DEPOT}/p4gf_config'
P4GF_CONFIG_REPO = '//{P4GF_DEPOT}/repos/{repo_name}/p4gf_config'
P4GF_CONFIG_REPO2 = '//{P4GF_DEPOT}/repos/{repo_name}/p4gf_config2'

P4GF_DEPOT_BRANCH_ROOT = "//{P4GF_DEPOT}/branches/{repo_name}/{branch_id}"

# branch-info files, separate from the versioned files that the branch stores.
# Nothing but branch-info files can be below this root.
P4GF_DEPOT_BRANCH_INFO_ROOT = "//{P4GF_DEPOT}/branch-info"

P4GF_CHANGELIST_DATA_FILE = '//{P4GF_DEPOT}/changelists/{repo_name}/{change_num}'

# Placed in change description when importing from Git to Perforce.
P4GF_IMPORT_HEADER = 'Imported from Git'
P4GF_IMPORT_HEADER_OLD = 'Git Fusion additional data:'
P4GF_DESC_KEY_AUTHOR = 'Author'      # Do not change: required by git-fast-import
P4GF_DESC_KEY_COMMITTER = 'Committer'   # Do not change: required by git-fast-import
P4GF_DESC_KEY_PUSHER = 'Pusher'
P4GF_DESC_KEY_SHA1 = 'sha1'
P4GF_DESC_KEY_PUSH_STATE = 'push-state'
P4GF_DESC_KEY_DEPOT_BRANCH_ID = 'depot-branch-id'
P4GF_DESC_KEY_CONTAINS_P4_EXTRA = 'contains-p4-extra'
P4GF_DESC_KEY_GITLINK = 'gitlink'
P4GF_DESC_KEY_PARENTS = 'parents'
P4GF_DESC_KEY_PARENT_CHANGES = 'parent-changes'
P4GF_DESC_KEY_PARENT_BRANCH = 'parent-branch'
P4GF_DESC_KEY_GHOST_OF_SHA1 = 'ghost-of-sha1'
P4GF_DESC_KEY_GHOST_OF_CHANGE_NUM = 'ghost-of-change-num'
P4GF_DESC_KEY_GHOST_PRECEDES_SHA1 = 'ghost-precedes-sha1'

# Placed in commit message when exporting from Perforce to Git
P4GF_EXPORT_HEADER = 'Copied from Perforce'
P4GF_DESC_KEY_CHANGE = 'Change'
P4GF_DESC_KEY_IDENTITY = 'Identity'

# 'git clone' of these views (or pulling or fetching or pushing) runs special commands
P4GF_UNREPO_INFO = '@info'      # Returns our version text
P4GF_UNREPO_LIST = '@list'      # Returns list of repos visible to user
P4GF_UNREPO_HELP = '@help'      # Returns contents of help.txt, if present
P4GF_UNREPO_FEATURES = '@features'  # Reports enabled state of features
P4GF_UNREPO_WAIT = '@wait'      # wait for lock to be released
P4GF_UNREPO_STATUS = '@status'    # report status of push operation
P4GF_UNREPO_PROGRESS = '@progress'  # report status changes while waiting
P4GF_UNREPO_CONFIG = '@config'    # dump configuration of repository
P4GF_UNREPO_PS = '@ps'        # list active pull/push operations
P4GF_UNREPO_CL = '@cl'        # find SHA1/changelist
P4GF_UNREPO = [
    P4GF_UNREPO_INFO,
    P4GF_UNREPO_LIST,
    P4GF_UNREPO_HELP,
    P4GF_UNREPO_FEATURES,
    P4GF_UNREPO_WAIT,
    P4GF_UNREPO_STATUS,
    P4GF_UNREPO_PROGRESS,
    P4GF_UNREPO_CONFIG,
    P4GF_UNREPO_PS,
    P4GF_UNREPO_CL
]
# -----------------------------------------------------------------------------
#                 Begin block copied to both p4gf_const.py
#                 and p4gf_submit_trigger.py.
#
# Atomic Push
#
# Atomic view locking requires special p4keys and users to insert Reviews into
# the user spec Each Git Fusion server has its own lock.
#
P4GF_REVIEWS_GF = 'git-fusion-reviews-'  # Append GF server_id.
P4GF_REVIEWS__NON_GF = P4GF_REVIEWS_GF + '-non-gf'
P4GF_REVIEWS__ALL_GF = P4GF_REVIEWS_GF + '-all-gf'
P4GF_REVIEWS_NON_GF_SUBMIT = 'git-fusion-non-gf-submit-'
P4GF_REVIEWS_NON_GF_RESET = 'git-fusion-non-gf-'
DEBUG_P4GF_REVIEWS__NON_GF = 'DEBUG-' + P4GF_REVIEWS__NON_GF
DEBUG_SKIP_P4GF_REVIEWS__NON_GF = 'DEBUG-SKIP-' + P4GF_REVIEWS__NON_GF
P4GF_REVIEWS_SERVICEUSER = P4GF_REVIEWS_GF + '{0}'
NON_GF_REVIEWS_BEGIN_MARKER_PATTERN = '//GF-{0}/BEGIN'
NON_GF_REVIEWS_END_MARKER_PATTERN = '//GF-{0}/END'
P4GF_REVIEWS_COMMON_LOCK = 'git-fusion-reviews-common-lock'
P4GF_REVIEWS_COMMON_LOCK_OWNER = 'git-fusion-reviews-common-lock-owner'
P4GF_REMAPPED = '//{P4GF_DEPOT}/remapped/{repo_name}'

# Is the Atomic Push submit trigger installed and at the correct version?
#
P4GF_P4KEY_TRIGGER_VERSION = 'git-fusion-submit-trigger-version'
P4GF_TRIGGER_VERSION = '2017.2-001'
P4GF_P4KEY_LOCK_USER = 'git-fusion-user-{user_name}-lock'

#
#                 End block copied to both p4gf_const.py
#                 and p4gf_submit_trigger.py.
# -----------------------------------------------------------------------------
P4GF_LOCKED_BY_MSG = "Files in the push are locked by '{user}'"

NULL_COMMIT_SHA1 = '0' * 40
EMPTY_TREE_SHA1 = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'
EMPTY_BLOB_SHA1 = 'e69de29bb2d1d6434b8b29ae775ad8c2e48c5391'
GITATTRIBUTES = '.gitattributes'

# Git-Swarm reviews: Git branch name prefix for any Swarm review.
P4GF_GIT_SWARM_REF_PREFIX_SHORT = "review/"
P4GF_GIT_SWARM_REF_PREFIX_FULL = "refs/heads/review/"

# File added in rare case when Git commit's own ls-tree is empty
# and does not differ from first-parent (if any).
P4GF_EMPTY_CHANGELIST_PLACEHOLDER = '.p4gf_empty_changelist_placeholder'

# 'p4 client' options for any client spec we create.
CLIENT_OPTIONS = 'allwrite clobber nocompress locked nomodtime normdir'

NO_SUCH_CLIENT_DEFINED = "No such client '{}' defined"
NO_SUCH_REPO_DEFINED = "No such repo '{}' defined"
NO_REPO_FROM_CLIENT_MSG_TEMPLATE = """
{nop4client}To define a Git Fusion repo:
* create a Git Fusion repo configuration file
  then check it into //P4GF_DEPOT/repos/{repo_name}/p4gf_config, or
* ask your Git Fusion administrator to
  create a Git Fusion repo configuration file
  then specify it on the command line to p4gf_init_repo.py --config <file>,
  run on the Git Fusion server
"""
NO_REPO_MSG_TEMPLATE = """
Git Fusion repo '{repo_name}' does not exist.
{nop4client}To define a Git Fusion repo:
* create a Perforce client spec '{repo_name_p4client}', or
* create a Git Fusion repo configuration file
  then check it into //P4GF_DEPOT/repos/{repo_name}/p4gf_config, or
* ask your Git Fusion administrator to
  create a Git Fusion repo configuration file
  then specify it on the command line to p4gf_init_repo.py --config <file>,
  run on the Git Fusion server
"""
DELETED_P4GF_CONFIG_MSG_TEMPLATE = """
The Git Fusion config file for repo '{repo_name}' has been deleted:
  //P4GF_DEPOT/repos/{repo_name}/p4gf_config
To define this Git Fusion repo:
* re-create the Git Fusion repo configuration file
  then check it into //P4GF_DEPOT/repos/{repo_name}/p4gf_config, or
* ask your Git Fusion administrator to
  create a Git Fusion repo configuration file
  then specify it on the command line to p4gf_init_repo.py --config <file>,
  run on the Git Fusion server
"""
MISSING_P4GF_CONFIG_MSG_TEMPLATE = """
The Git Fusion config file for repo '{repo_name}' has been obliterated:
  //P4GF_DEPOT/repos/{repo_name}/p4gf_config
* Contact your Git Fusion administrator to
  address this situation and configure this repo.
"""
EMPTY_VIEWS_MSG_TEMPLATE = """
Git Fusion repo '{repo_name}' cannot be created.
The views/exclusions for a branch/client allow no paths.
To define a Git Fusion repo:
* create a Perforce client spec '{repo_name_p4client}', or
* create a Git Fusion repo configuration file
  then check it into //P4GF_DEPOT/repos/{repo_name}/p4gf_config, or
* ask your Git Fusion administrator to
  create a Git Fusion repo configuration file
  then specify it on the command line to p4gf_init_repo.py --config <file>,
  run on the Git Fusion server
"""
# -- substitution-savvy functions ---------------------------------------------


def objects_root():
    """Return //P4GF_DEPOT/objects."""
    return P4GF_OBJECTS_ROOT.format(P4GF_DEPOT=P4GF_DEPOT)

# Not officially tested or supported, but quite useful: import any environment
# variables starting P4GF_ as overrides to replace the above constants.
#
# h/t to Ravenbrook for the feature.
# https://github.com/Ravenbrook/perforce-git-fusion/commit/5cace4df621b91ba8b3b20059400af5a3e0837f2
#
# Commented out until we can find all the places in our test machinery that set
# P4GF_ environment variables that break the automated tests.
#
# import os
# locals().update({ key:value
#                   for key, value in os.environ.items()
#                   if key.startswith('P4GF_')})
