#! /usr/bin/env python3.3
"""Tell me about 'p4 filelog' 'how' values."""

# All known 'how' values.
# Useful mostly as copy-and-paste fodder for later lookup dicts.

ALL = [
    'add from', 'add into',
    'branch from', 'branch into',
    'copy from', 'copy into',
    'delete from', 'delete into',
    'edit from', 'edit into',
    'ignored', 'ignored by',      # Note different preposition.
    'merge from', 'merge into',
    'move from', 'move into'
]

# Integrating from some other path. 'file' contains source.
FROM = [
    'add from',
    'branch from',
    'copy from',
    'delete from',
    'edit from',
    'ignored',         # Note different preposition.
    'merge from',
    'move from'
]

# If enable-cherry-pick is enabled, only branch operations
# are eligible to be reflected into git. All other integ actions are ignored
# resulting in git edits rather then git merge commits.
FROM_WITH_CHERRY_PICK = [
    'branch from'
]


def is_from(how, cherry_pick_enabled):
    """Return True if action specify an integ-like action _from_ somewhere.

    If so, then 'p4 filelog's corresponding 'file' value will be a source.
    """
    if cherry_pick_enabled:
        return how in FROM_WITH_CHERRY_PICK
    else:
        return how in FROM


# Integrating to some other path. 'file' contains destination.
TO = [
    'add into',
    'branch into',
    'copy into',
    'delete into',
    'edit into',
    'ignored by',      # Note different preposition.
    'merge into',
    'move into'
]


def is_to(how):
    """Return True if action specify an integ-like action _to_ somewhere.

    If so, then 'p4 filelog's corresponding 'file' value will be a destination.
    """
    return how in TO
