#! /usr/bin/env python3.3
"""ObjectTypeList."""

from collections import Sequence


class ObjectTypeList(Sequence):

    """Immutable list of ObjectType instances.

    Compares to other instances using sha1.
    Simply wraps an instance of the built-in list type.
    """

    def __init__(self, sha1, ot_list):
        """Construct ObjectTypeList."""
        self.sha1 = sha1
        self.ot_list = ot_list

    def __hash__(self):
        """Return hash code."""
        return hash(self.sha1)

    def __eq__(self, other):
        """Return True if equal."""
        return self.sha1 == other.sha1

    def __ne__(self, other):
        """Return True if not equal."""
        return self.sha1 != other.sha1

    def __ge__(self, other):
        """Return True if greater or equal."""
        return self.sha1 >= other.sha1

    def __gt__(self, other):
        """Return True if greater."""
        return self.sha1 > other.sha1

    def __le__(self, other):
        """Return True if less than or equal."""
        return self.sha1 <= other.sha1

    def __lt__(self, other):
        """Return True if less than."""
        return self.sha1 < other.sha1

    def __getitem__(self, index):
        """Return object at index."""
        return self.ot_list[index]

    def __len__(self):
        """Return length of collection."""
        return len(self.ot_list)

    def __str__(self):
        """Return str of this."""
        return "ObjectTypeList[{}]".format(self.sha1[:7])
